import os

import axipy
from PySide2 import QtGui
from PySide2.QtCore import QPoint
from PySide2.QtGui import Qt, QCursor, QMouseEvent, QPainter, QPaintEvent
from axipy import MapTool, Point, Rectangle, Style, Line, Pnt, ActiveToolPanel, ObserverManager, MapView, Feature, \
    LineString

from ru_axioma_gis_add_geo_tools.tools.line_pairing.tools.map_add_tools import find_object_in_map, createSegmentInfo
from ru_axioma_gis_add_geo_tools.tools.line_pairing.tools.pairing_geo_object import GeometryPairing, TypeSegment
from ru_axioma_gis_add_geo_tools.tools.line_pairing.tools.save_params import SaveParamsBevel
from ru_axioma_gis_add_geo_tools.tools.line_pairing.widget_tool import ToolPairing


class PairingMapTool(MapTool):
    def __init__(self,param_saver:SaveParamsBevel):
        super().__init__()
        self.__cls_params=param_saver
        self.__curent_index_segment=0
        self.__segments=[None,None]
        self.__params=self.__cls_params.load()
        self.__default_style=Style.from_mapinfo("Pen (4, 59, 16711680)")
        self.__widget_panel=ToolPairing(self.__params)
        service = ActiveToolPanel()
        self.tool_panel = service.make_custom("Скругление/Скос", observer_id=ObserverManager.Editable, widget=self.__widget_panel.widget)
        #self.tool_panel.deactivated.connect(self.__deactivate_widget)
        self.tool_panel.panel_was_closed.connect(self.__close_widget)
        self.__is_view_panel=False
        self.__cls_Pair=None
        self.__widget_panel.signal_run.connect(self.__run)
    def load(self) -> None:
        path_cursor=os.path.join(os.path.dirname(os.path.realpath(__file__)), 'icons',
                                         'сursor_big_pair.png')
        self.cursor=QtGui.QCursor(QtGui.QPixmap(path_cursor))
        #self.cursor=QCursor(Qt.CrossCursor)
    def mousePressEvent(self, event: QMouseEvent) -> bool:
        if not(event.button() == Qt.LeftButton):
            return self.BlockEvent
        if not  self.__is_view_panel :
            try:
                self.tool_panel.activate()
            except:
                self.__widget_panel=ToolPairing(self.__params)
                self.tool_panel.set_widget(self.__widget_panel)
                self.tool_panel.activate()
            self.__is_view_panel=True
        #device_point = event.pos()
        #point=Point(device_point.x(),device_point.y())
        #map_point=self.to_scene(device_point)
        self.__getSegment(event.pos())
        self.__isTwoSegment()
        return self.BlockEvent
    def __getSegment(self,device_point):
        dd= int(axipy.CurrentSettings.SensitiveMouse/2+0.5)
        s_x_min=device_point.x()-dd
        s_y_min=device_point.y()-dd
        s_x_max = device_point.x() + dd
        s_y_max = device_point.y() + dd
        sp_0=QPoint(s_x_min,s_y_min)
        sp_1 = QPoint(s_x_min, s_y_max)
        sp_2 = QPoint(s_x_max, s_y_max)
        sp_3 = QPoint(s_x_max, s_y_min)
        mp_0=self.to_scene(sp_0)
        mp_1 = self.to_scene(sp_1)
        mp_2 = self.to_scene(sp_2)
        mp_3 = self.to_scene(sp_3)
        rect_sel=Rectangle(mp_0.x,mp_0.y,mp_2.x,mp_2.y,cs=self.view.coordsystem)
        ft_obj,geo_intersect,name_table=find_object_in_map(self.view.map,rect_sel)
        if ft_obj is not None:
            segment=createSegmentInfo(ft_obj, geo_intersect, name_table,rect_sel)
            if segment.type_segment==TypeSegment.Start or segment.type_segment==TypeSegment.End:
                self.__widget_panel.setEditableSegmet(self.__curent_index_segment)
            else:
                self.__widget_panel.setEditableSegmet(self.__curent_index_segment,False,False)


            self.__segments[self.__curent_index_segment]=segment
            self.__curent_index_segment=self.__curent_index_segment+1
            if self.__curent_index_segment>=2:
                self.__curent_index_segment=0
            self.redraw()
    def __isTwoSegment(self):
        count=0
        for seg in self.__segments:
            if seg is None:
                continue
            count=count+1
        if count==2:
            self.__cls_Pair = GeometryPairing(self.__segments[0], self.__segments[1])
            if not self.__cls_Pair.isIntersect:
                axipy.Notifications.push('Скругление/Скос', 'Выбранные сегменты не пересекаются', axipy.Notifications.Warning)
                self.__cls_Pair=None
                return
            self.__widget_panel.setEnabledRun()



    def paintEvent(self, event: QPaintEvent, painter: QPainter):
        painter.save()
        self.__drawSegments(painter)
        painter.restore()
        super().paintEvent(event, painter)
    def __lineToScreen(self,line):
        pnt_start=self.to_device(line.begin)
        pnt_end=self.to_device(line.end)
        return Line(Pnt(pnt_start.x(),pnt_start.y()),Pnt(pnt_end.x(),pnt_end.y()))
    def __drawSegments(self,painter):
        if len(self.__segments)==0:
            return
        for segment in self.__segments:
            if segment is None:
                continue
            line_screen=self.__lineToScreen(segment.line.reproject(self.view.coordsystem))
            self.__widget_panel.style_sergment.draw(line_screen,painter)
    def unload(self) -> None:
        if self.tool_panel.activated:
            params = self.__widget_panel.getParams()
            self.__cls_params.save(params)
            self.__is_view_panel = False
            self.tool_panel.deactivate()


    def __run(self,params):
        geo_segment_1= geo_segment_2=None
        if params['method']=='pairing':
            geo_segment_1,geo_segment_2,circle_segment=self.__cls_Pair.build(params['radius'])
        if params['method'] == 'bevel':
            dist_1 = params['dist_1']
            dist_2 = params['dist_2']
            geo_segment_1, geo_segment_2, bevel_line = self.__cls_Pair.run_bevel(dist_1, dist_2)
            circle_segment=bevel_line
        out_table = None
        if isinstance(axipy.view_manager.active, MapView) and axipy.view_manager.active.editable_layer is not None:
            out_table = axipy.view_manager.active.editable_layer.data_object
                # cs_tab_sorce_segmet_1= axipy.data_manager.find(self.__cls_Pair.segment_1.name_table).coordsystem
                # circle_segment.coordsystem=out_table.coordsystem
                # out_geometry=LineString(circle_segment.points,cs=cs_tab_sorce_segmet_1)
            out_geometry = circle_segment
            out_geometry.coordsystem = self.__cls_Pair.BaseCoordSystem
            ft_new = Feature(geometry=out_geometry)
            out_table.insert(ft_new)
            if geo_segment_1 is not None and params['edit_seg_1']:
                '''Редактируем 1 сегмент'''
                geo_segment_1.coordsystem = self.__cls_Pair.BaseCoordSystem
                updateGeometrySegment(self.__cls_Pair.segment_1, geo_segment_1)
            if geo_segment_2 is not None and params['edit_seg_2']:
                '''Редактируем 2 сегмент'''
                geo_segment_2.coordsystem = self.__cls_Pair.BaseCoordSystem
                updateGeometrySegment(self.__cls_Pair.segment_2, geo_segment_2)

            jkl=0
        self.__segments=[None,None]
        self.redraw()
    def __deactivate_widget(self):
        jkl = 0
        self.__is_view_panel = False
        #self.__segments=[None,None]
        #self.redraw()

    def __close_widget(self):
        jkl = 0
        self.__is_view_panel=False

def updateGeometry(source_geometry,type_segment,new_segment):
    if isinstance(source_geometry, axipy.Line):
        return new_segment
    if type_segment==TypeSegment.Start:
        source_geometry.points[0]=new_segment.begin
        return source_geometry
    else:
        count_pnt=len(source_geometry.points)
        source_geometry.points[count_pnt-1] = new_segment.end
        return source_geometry

def updateGeometrySegment(segment,new_geo):
    tab=axipy.data_manager.find(segment.name_table)
    fts=list(tab.itemsByIds([segment.ft_id]))
    ft_update=fts[0]
    update_geometry=updateGeometry(ft_update.geometry,segment.type_segment,new_geo)
    ft_update.geometry=update_geometry
    tab.update(ft_update)


