from axipy import Map, Rect, ListLayers, Table, GeometryClass, Rectangle, Line, LineString, Polygon
from shapely.io import from_wkb, to_wkb

from axipy import Geometry
from ru_axioma_gis_add_geo_tools.tools.line_pairing.tools.pairing_geo_object import SegmentDescription, TypeSegment


def find_geometry_intersectRect(table,rect_bound:Geometry):
    bound=rect_bound.bounds
    geo_rect=rect_bound
    if rect_bound.coordsystem!=table.coordsystem:
        geo_rect=rect_bound.reproject(table.coordsystem)
        bound=geo_rect.bounds
    geo_obj=None
    for ft in table.items(bound):
        geometry=ft.geometry
        if not(isinstance(geometry,Line) or isinstance(geometry,LineString) or isinstance(geometry,Polygon)):
            continue
        geo_line_string=geometry
        if isinstance(geometry,Polygon):
            geo_line_string=geometry.to_linestring()
        geo_intersect=geo_line_string.intersection(geo_rect)
        if geo_intersect is not None:
            if isinstance(geo_intersect,LineString) or isinstance(geo_intersect,Line):

                return ft,geo_intersect
    return None,None


def find_in_layers(layers:ListLayers,bound):
    for layer in layers:
        if not layer.visible:
            continue
        if isinstance(layer,ListLayers):
            ft_geometry_intersect,geometry_intersect=find_in_layers(layer,bound)
            if ft_geometry_intersect is not None:
                return ft_geometry_intersect,geometry_intersect

        data_source=layer.data_object
        if not isinstance(data_source,Table):
            continue
        if data_source.coordsystem.lat_lon:
            continue
        if not data_source.is_spatial:
            continue
        class_geometry=data_source.class_geometries()
        is_line=GeometryClass.Lines in class_geometry
        if not is_line:
            is_line=GeometryClass.Polygons
        if not is_line:
            continue
        ft_geometry_intersect,geometry_intersect=find_geometry_intersectRect(data_source,bound)
        if ft_geometry_intersect is not None:
            return ft_geometry_intersect,geometry_intersect,data_source.name
    return None,None,None
def find_object_in_map(map:Map,bound:Rectangle):
    layers=map.layers
    ft_obj,geo_intersect,name_tab=find_in_layers(layers,bound)
    return ft_obj,geo_intersect,name_tab

def findSegment(line_string,bound):
    count=len(line_string.points)
    pt_0=line_string.points[0]
    for i in range(1,count):
        pt_1=line_string.points[i]
        temp_line=Line(pt_0,pt_1,bound.coordsystem)
        if temp_line.intersects(bound):
            type_segment=TypeSegment.Start
            if i>1 and i<(count-1):
                type_segment=TypeSegment.Interior
            if i==(count-1):
                type_segment=TypeSegment.End
            return temp_line,type_segment
        pt_0=pt_1
    return None,None
def createSegmentInfo(ft_select,geo_intersect,name_table,bound):
    geometry=ft_select.geometry
    type_geometry=geometry.type
    if isinstance(geometry,Polygon):
        geometry=geometry.to_linestring()

    geo_rect = bound
    if bound.coordsystem != geometry.coordsystem:
        geo_rect = bound.reproject(geometry.coordsystem)
        #bound = geo_rect.bounds
    point_select = geo_intersect.centroid()
    if isinstance(geometry,Line):

        shema_description=SegmentDescription(name_table,ft_select.id,type_geometry,geometry,point_select,TypeSegment.Start)
        return shema_description
    if isinstance(geometry,LineString):
        segment_intersect,type_segment=findSegment(geometry,geo_rect)
        if segment_intersect is None:
            return None
        shema_description = SegmentDescription(name_table,ft_select.id, type_geometry, segment_intersect, point_select, type_segment)
        return shema_description
    for geo_line_string in geometry:
        segment_intersect, type_segment = findSegment(geo_line_string, geo_rect)
        if segment_intersect is None:
            continue
        shema_description = SegmentDescription(name_table,ft_select.id, type_geometry, segment_intersect, point_select,
                                               type_segment)
        return shema_description
    return None
def shapely_to_minimum_rotated_rectangle(geometrys,coordsys):
    sh_geometrys=[]
    wkb_geo = geometrys.to_wkb()
    s_geo = from_wkb(wkb_geo)
    new_geo=s_geo.minimum_rotated_rectangle
    new_axi=Geometry.from_wkb(to_wkb(new_geo), coordsys)
    return new_axi




