from enum import Enum

import axipy
from PySide2.QtCore import QLineF
from axipy import Line, Pnt, Rect, Geometry, LineString, LineDirection, Ellipse, Polygon, Point, LinearUnits, \
    CoordSystem

from ru_axioma_gis_add_geo_tools.tools.parallel_linestring.create_parallel import TypeOffset, shapaly_offset
def pointToPnt(point:Point):
    return Pnt(point.x,point.y)
def calcPerpendicularFromPoint(line:Line,p3:Pnt):
    x1=line.begin.x
    y1=line.begin.y
    x2=line.end.x
    y2=line.end.y
    a=x2-x1
    b=y2-y1
    a_sq = a * a
    b_sq = b * b
    ab = a * b
    a_sq_plus_b_sq = a_sq + b_sq
    x3=p3.x
    y3=p3.y
    x = (a_sq * x3 + ab * (y3 - y1) + b_sq * x1) / a_sq_plus_b_sq
    y = (a_sq * y1 + ab * (x3 - x1) + b_sq * y3) / a_sq_plus_b_sq
    return Pnt(x,y)
def PointToPnt(point):
    if isinstance(point,Point):
        return Pnt(point.x,point.y)
    return point
def getLineOrientationToPoint(line,point:Pnt):
    dist_1,angle=Geometry.distance_by_points(line.begin,point)
    dist_2, angle = Geometry.distance_by_points(line.end, point)
    if dist_1>dist_2:
        return line,Line(line.begin,point),False
    return Line(line.end, line.begin),Line(line.end,point),True
def lineIntersect(l1_point,l2_point):
    line_1=QLineF(l1_point[0].x,l1_point[0].y,l1_point[1].x,l1_point[1].y)
    line_2=QLineF(l2_point[0].x,l2_point[0].y,l2_point[1].x,l2_point[1].y)
    type_intersect,point_intersect=line_1.intersect(line_2)
    return type_intersect
def fixIntersect(l1_point,l2_point):
    geo_line1=Line(l1_point[0],l1_point[1])
    pnt_1=Pnt(l2_point[0].x,l2_point[0].y)
    pnt_2 = Pnt(l2_point[1].x, l2_point[1].y)
    geo_line2 = Line(pnt_1, pnt_2)
    isIntersect=geo_line1.intersects(geo_line2)
    if isIntersect:
        return QLineF.BoundedIntersection
    return QLineF.UnboundedIntersection
def getExtendLineToBound(bound:Rect,line:Line):
    q_line=QLineF(line.begin.x,line.begin.y,line.end.x,line.end.y)
    list_line_bound=[]
    list_line_bound.append(QLineF(bound.xmin,bound.ymin,bound.xmax,bound.ymin))
    list_line_bound.append(QLineF(bound.xmin, bound.ymax, bound.xmax, bound.ymax))
    list_line_bound.append(QLineF(bound.xmin, bound.ymin, bound.xmin, bound.ymax))
    list_line_bound.append(QLineF(bound.xmax, bound.ymin, bound.xmax, bound.ymax))
    points=[]
    for lb in list_line_bound:
        type_intersect,point_intersect=lb.intersect(q_line)
        if type_intersect==QLineF.NoIntersection:
            continue


        isExist=False
        for pnt in points:
            if pnt==point_intersect:
                isExist=True
                break
        if not isExist:
            points.append(point_intersect)
    if len(points)<2:
        return line
    return Line(Pnt(points[0].x(),points[0].y()),Pnt(points[1].x(),points[1].y()))
def toQLinef(geometry:Geometry):
    source_geo=geometry
    if isinstance(geometry,Line):
        source_geo=geometry.to_linestring()
    count_point=len(source_geo.points)
    return QLineF(source_geo.points[0].x,source_geo.points[0].y,source_geo.points[count_point-1].x,source_geo.points[count_point-1].y)
class TypeSegment(Enum):
    Start=1
    End=2
    Interior=3 # внутренний

class SegmentDescription:
    def __init__(self,name_table,ft_id,type_geometry,line:Line,point_select,type_segment,editable=True):
        self.name_table=name_table
        self.ft_id=ft_id
        self.geometry_type=type_geometry
        self.line=line
        self.pnt_select=point_select
        self.type_segment=type_segment
        self.editable=editable
        if type_segment==TypeSegment.Interior:
            self.editable=False
        self.__typeIntersect = QLineF.NoIntersection
class GeometryPairing:
    def __init__(self,segmet_1:SegmentDescription,segmet_2:SegmentDescription):
        self.__segment_1=segmet_1
        self.__segment_2=segmet_2
        self.__point_intersect=None
        #self.__prepare()
        self.__base_cs=None
        self.__prepareCoordSys()
        self.__prepareAxi()
    def __prepareCoordSys(self):
        self.__base_cs=CoordSystem.from_wkt('LOCAL_CS["Nonearth",UNIT["Meter",1],AXIS["Easting",EAST],AXIS["Northing",NORTH]]')
        self.__coordsys_1=self.__segment_1.line.coordsystem
        self.__coordsys_2=self.__segment_2.line.coordsystem
        if self.__coordsys_1.non_earth and self.__coordsys_2.non_earth:
            if self.__coordsys_1.unit==LinearUnits.m and self.__coordsys_2.unit==LinearUnits.m:
                return
            if self.__coordsys_1.unit!=LinearUnits.m:
                self.__segment_1.line=self.__segment_1.line.reproject(self.__base_cs)
                self.__segment_1.pnt_select = pointToPnt(Point(self.__segment_1.pnt_select.x,self.__segment_1.pnt_select.y,cs=self.__coordsys_1).reproject(self.__base_cs))
            if self.__coordsys_2.unit!=LinearUnits.m:
                self.__segment_2.line=self.__segment_2.line.reproject(self.__base_cs)
                self.__segment_2.pnt_select = pointToPnt(
                    Point(self.__segment_2.pnt_select.x, self.__segment_2.pnt_select.y, cs=self.__coordsys_2).reproject(
                        self.__base_cs))
            return
        self.__base_cs=self.__coordsys_1
        if self.__base_cs!=self.__coordsys_2:
            self.__segment_2.line = self.__segment_2.line.reproject(self.__base_cs)
            self.__segment_2.pnt_select = pointToPnt(
                Point(self.__segment_2.pnt_select.x, self.__segment_2.pnt_select.y, cs=self.__coordsys_2).reproject(
                    self.__base_cs))

    def __prepare(self):
        line_1 = QLineF(self.__segment_1.line.begin.x, self.__segment_1.line.begin.y, self.__segment_1.line.end.x, self.__segment_1.line.end.y)
        line_2 = QLineF(self.__segment_2.line.begin.x, self.__segment_2.line.begin.y, self.__segment_2.line.end.x, self.__segment_2.line.end.y)
        result = line_1.intersect(line_2)
        self.__typeIntersect=result[0]
        if self.__typeIntersect==QLineF.NoIntersection:
            self.__point_intersect=None
            return
        self.__point_intersect=Pnt(result[1].x(),result[1].y())
    @property
    def segment_1(self):
        return self.__segment_1

    @property
    def segment_2(self):
        return self.__segment_2
    @property
    def isIntersect(self):
        if self.__point_intersect is not None:
            return True
        return False

    def __prepareAxi(self):
        self.__commonBound()
        self.__virtual_line_1=getExtendLineToBound(self.__com_bound,self.__segment_1.line)
        self.__virtual_line_2 = getExtendLineToBound(self.__com_bound, self.__segment_2.line)
        self.__point_intersect=None
        self.__virtual_line_1_qt=toQLinef(self.__virtual_line_1)
        self.__virtual_line_2_qt = toQLinef(self.__virtual_line_2)
        self.__typeIntersect,pnt_intersect=self.__virtual_line_1_qt.intersect(self.__virtual_line_2_qt)
        self.__point_intersect=Pnt(pnt_intersect.x(),pnt_intersect.y())

    def __commonBound(self):
        self.__com_bound=self.__segment_1.line.bounds
        self.__com_bound=self.__com_bound.merge(self.__segment_2.line.bounds)
    @property
    def BaseCoordSystem(self):
        return self.__base_cs
    def build(self,radius_value:axipy.UnitValue):
        radius=radius_value.to_unit(self.__base_cs.unit).value
        if self.__segment_1.geometry_type==axipy.cpp_core_geometry.Type.Line and self.__segment_2.geometry_type==axipy.cpp_core_geometry.Type.Line:
            ''' оба обекта линии '''
            if self.__typeIntersect == QLineF.UnboundedIntersection:
                return self.__UnboundedIntersect(radius)
            if self.__typeIntersect == QLineF.BoundedIntersection:
                return self.__BoundedIntersect(radius)
        if self.__typeIntersect == QLineF.BoundedIntersection:
            return self.__BoundedIntersect(radius)
        if self.__typeIntersect == QLineF.UnboundedIntersection:
            return self.__UnboundedIntersect(radius)
        return None,None,None
    def run_bevel(self,dist_1:axipy.UnitValue,dist_2:axipy.UnitValue):
        dist_1_base_cs=dist_1.to_unit(self.__base_cs.unit).value
        out_geometry_1,point_bevel_1=self.__bevel_segment(self.__segment_1,dist_1_base_cs)
        dist_2_base_cs = dist_2.to_unit(self.__base_cs.unit).value
        out_geometry_2, point_bevel_2 = self.__bevel_segment(self.__segment_2, dist_2_base_cs)
        out_segment=Line(point_bevel_1,point_bevel_2,cs=self.__base_cs)
        return out_geometry_1,out_geometry_2,out_segment
    def __bevel_segment(self,segment,dist:float):
        if self.__typeIntersect ==QLineF.UnboundedIntersection:
            return self.__bevel_UnboundedIntersection(segment,dist)
        else:
            return self.__bevel_BoundedIntersection(segment,dist)
    def __bevel_BoundedIntersection(self,segment,dist):
        dist_to_point_intersect,angle=Geometry.distance_by_points(pointToPnt(self.__point_intersect),pointToPnt(segment.pnt_select),self.__base_cs)
        pnt_bevel=Geometry.point_by_azimuth(pointToPnt(self.__point_intersect),angle,dist,self.__base_cs)
        if (not segment.editable) or segment.type_segment==TypeSegment.Interior:
            return None,pnt_bevel
        int_angle=int(angle+0.5)
        '''определяем первую точку'''
        dist_pt_1,angle_1=Geometry.distance_by_points(segment.line.begin,pointToPnt(self.__point_intersect),self.__base_cs)
        dist_pt_2, angle_2 = Geometry.distance_by_points(segment.line.end, pointToPnt(self.__point_intersect), self.__base_cs)

        type_start_pnt=TypeSegment.End
        if int_angle==int(angle_2+0.5):
            type_start_pnt=TypeSegment.Start
        out_geometry=None
        if type_start_pnt==TypeSegment.Start:
            out_geometry=Line(segment.line.begin,pnt_bevel,cs=self.__base_cs)
        else:
            out_geometry = Line(segment.line.end, pnt_bevel, cs=self.__base_cs)
        if segment.geometry_type==axipy.cpp_core_geometry.Type.Line:
            return out_geometry,pnt_bevel
        if segment.type_segment==TypeSegment.Start and type_start_pnt==TypeSegment.End:
            return out_geometry, pnt_bevel
        if segment.type_segment==TypeSegment.Start and type_start_pnt==TypeSegment.Start:
            return None, pnt_bevel
        if segment.type_segment==TypeSegment.End and type_start_pnt==TypeSegment.End:
            return None, pnt_bevel
        if segment.type_segment==TypeSegment.End and type_start_pnt==TypeSegment.Start:
            return out_geometry, pnt_bevel
        return None,pnt_bevel
    def __bevel_UnboundedIntersection(self,segment,dist):
        dist_1,angle_1=Geometry.distance_by_points(segment.line.begin,self.__point_intersect)
        dist_2, angle_2 = Geometry.distance_by_points(segment.line.end, self.__point_intersect)
        nearest_pnt=segment.line.end
        type_point=TypeSegment.End
        if dist_1<dist_2:
            nearest_pnt = segment.line.begin
            type_point = TypeSegment.Start
        dits_z,angle=Geometry.distance_by_points(self.__point_intersect,segment.line.end)
        point_bevel=Geometry.point_by_azimuth(self.__point_intersect,angle,dist,self.__base_cs)
        out_geometry=None
        if not segment.editable or segment.type_segment==TypeSegment.Interior:
            return out_geometry,point_bevel
        if type_point==TypeSegment.Start:
            out_geometry=Line(point_bevel,segment.line.end,cs=self.__base_cs)
        else:
            out_geometry=Line(segment.line.begin,point_bevel,cs=self.__base_cs)
        return out_geometry, point_bevel


    def __UnboundedIntersect(self,radius):
        '''Линии пресекаются ,но не явно'''
        new_line1,line_1p,reverse_points_1=getLineOrientationToPoint(self.__segment_1.line,self.__point_intersect)
        new_line2,line_2p,reverse_points_2=getLineOrientationToPoint(self.__segment_2.line,self.__point_intersect)
        temp_poly=LineString([new_line1.begin,self.__point_intersect,new_line2.begin])
        line_direction=temp_poly.get_line_direction()
        if line_direction==LineDirection.Clockwise:
            line_1_parallel_p=shapaly_offset(new_line1,radius,TypeOffset.Rigth)
            line_2_parallel_p = shapaly_offset(new_line2, radius, TypeOffset.Left)
            line_1_parallel=line_1_parallel_p[1]
            line_2_parallel=line_2_parallel_p[0]
        else:
            line_1_parallel_p = shapaly_offset(new_line1, radius, TypeOffset.Left)
            line_2_parallel_p = shapaly_offset(new_line2, radius, TypeOffset.Rigth)
            line_1_parallel = line_1_parallel_p[0]
            line_2_parallel = line_2_parallel_p[1]
        #point_intersect=line_1_parallel.intersection(line_2_parallel)
        qline_1=toQLinef(line_1_parallel)
        qline_2 = toQLinef(line_2_parallel)
        intersect_info=qline_1.intersect(qline_2)
        point_intersect=Pnt(intersect_info[1].x(),intersect_info[1].y())
        rect_circle=Rect(point_intersect.x-radius,point_intersect.y-radius,point_intersect.x+radius,point_intersect.y+radius)
        circl=Ellipse(rect_circle)
        circl.center=(point_intersect.x,point_intersect.y)
        circl.majorSemiAxis=radius
        circl.minorSemiAxis=radius
        poly_circle=circl.to_linestring()
        #circle_line1_intrsect=poly_circle.intersection(line_1p)
        circle_line1_intrsect=calcPerpendicularFromPoint(line_1p,point_intersect)
        #circle_line2_intrsect = poly_circle.intersection(line_2p)
        circle_line2_intrsect = calcPerpendicularFromPoint(line_2p,point_intersect)
        line_split_circle=Line(circle_line1_intrsect,circle_line2_intrsect)
        split_circle=poly_circle.split_by_polyline(line_split_circle)
        temp_polygon_=Polygon([new_line1.begin,new_line1.end,circle_line1_intrsect,circle_line2_intrsect,new_line2.end,new_line2.begin])
        split_circle_out=poly_circle.difference(temp_polygon_)
        split_circel_temp=poly_circle.split_by_polyline(temp_polygon_.to_linestring())
        if len(split_circel_temp)==2:
            centroid_1=pointToPnt(split_circle[0].centroid())
            centroid_2=pointToPnt(split_circle[1].centroid())
            dist_1,angle=Geometry.distance_by_points(self.__point_intersect,centroid_1)
            dist_2,angle = Geometry.distance_by_points(self.__point_intersect, centroid_2)
            if dist_1<dist_2:
                split_circle_out=split_circle[0]
            else:
                split_circle_out = split_circle[1]
        out_line_1=None
        if reverse_points_1:
            out_line_1=Line(circle_line1_intrsect,self.__segment_1.line.end)
        else:
            out_line_1 = Line(self.__segment_1.line.begin,circle_line1_intrsect)
        out_line_2=None
        if reverse_points_2:
            out_line_2=Line(circle_line2_intrsect,self.__segment_2.line.end)
        else:
            out_line_2 = Line(self.__segment_2.line.begin,circle_line2_intrsect)
        return out_line_1,out_line_2,split_circle_out

    def __BoundedIntersectOld(self,radius):
        '''
        q_line_point_select=QLineF(self.__segment_1.pnt_select.x,self.__segment_1.pnt_select.y,self.__segment_2.pnt_select.x,self.__segment_2.pnt_select.y)
        result_l1=self.__virtual_line_1_qt.intersect(q_line_point_select)
        result_l2 = self.__virtual_line_2_qt.intersect(q_line_point_select)
        '''
        temp_poly = LineString([self.__segment_1.pnt_select, self.__point_intersect,self.__segment_2.pnt_select])
        line_direction = temp_poly.get_line_direction()
        new_line1=Line(self.__segment_1.pnt_select,self.__point_intersect)
        new_line2 = Line(self.__segment_2.pnt_select, self.__point_intersect)
        if line_direction == LineDirection.Clockwise:
            line_1_parallel_p = shapaly_offset(new_line1, radius, TypeOffset.Rigth)
            line_2_parallel_p = shapaly_offset(new_line2, radius, TypeOffset.Left)
            line_1_parallel = line_1_parallel_p[1]
            line_2_parallel = line_2_parallel_p[0]
        else:
            line_1_parallel_p = shapaly_offset(new_line1, radius, TypeOffset.Left)
            line_2_parallel_p = shapaly_offset(new_line2, radius, TypeOffset.Rigth)
            line_1_parallel = line_1_parallel_p[0]
            line_2_parallel = line_2_parallel_p[1]
        qline_1 = toQLinef(line_1_parallel)
        qline_2 = toQLinef(line_2_parallel)
        intersect_info = qline_1.intersect(qline_2)
        point_intersect = Pnt(intersect_info[1].x(), intersect_info[1].y())
        rect_circle = Rect(point_intersect.x - radius, point_intersect.y - radius, point_intersect.x + radius,
                           point_intersect.y + radius)
        circl = Ellipse(rect_circle)
        circl.center = (point_intersect.x, point_intersect.y)
        circl.majorSemiAxis = radius
        circl.minorSemiAxis = radius
        poly_circle = circl.to_linestring()
        # circle_line1_intrsect=poly_circle.intersection(line_1p)
        circle_line1_intrsect = calcPerpendicularFromPoint(self.__segment_1.line, point_intersect)
        # circle_line2_intrsect = poly_circle.intersection(line_2p)
        circle_line2_intrsect = calcPerpendicularFromPoint(self.__segment_2.line, point_intersect)
        line_split_circle = Line(circle_line1_intrsect, circle_line2_intrsect)
        split_circle = poly_circle.split_by_polyline(line_split_circle)

        temp_polygon_ = Polygon(
            [circle_line1_intrsect, self.__point_intersect,circle_line2_intrsect])

        centroid_1=split_circle[0].to_polygon().centroid()
        centroid_2 = split_circle[1].to_polygon().centroid()
        is_poly_1=temp_polygon_.contains(centroid_1)
        is_poly_2 = temp_polygon_.contains(centroid_2)
        '''
        split_circle_out = poly_circle.difference(temp_polygon_)
        '''
        split_circle_out=split_circle[0]
        if is_poly_2 and ( not is_poly_1):
            split_circle_out = split_circle[1]
        geometry_1_update=None
        if self.__segment_1.editable and self.__segment_1.geometry_type==axipy.cpp_core_geometry.Type.Line:
            ''' сегмент линия и редактируемый'''
            line_p2=(self.__segment_1.pnt_select,self.__segment_2.pnt_select)
            intersect_1=lineIntersect((self.__segment_1.line.begin,self.__point_intersect),line_p2)
            intersect_2 = lineIntersect((self.__segment_1.line.end, self.__point_intersect), line_p2)
            if intersect_1==QLineF.BoundedIntersection:
                ''' выходной ( изменяемый объект ) для первого сегмента от первой точки до точки пересечения'''
                geometry_1_update=Line(self.__segment_1.line.begin,circle_line1_intrsect,cs=self.__segment_1.line.coordsystem)
            if intersect_2==QLineF.BoundedIntersection:
                ''' выходной ( изменяемый объект ) для первого сегмента от точки пресечения до конечной точки'''
                geometry_1_update = Line( circle_line1_intrsect,self.__segment_1.line.end,
                                         cs=self.__segment_1.line.coordsystem)
            if intersect_1==QLineF.UnboundedIntersection and intersect_2==QLineF.UnboundedIntersection:
                ''' необходимо продить исходный отрезок'''
                dist_begin=Geometry.distance_by_points(self.__point_intersect,self.__segment_1.line.begin)
                dist_end = Geometry.distance_by_points(self.__point_intersect, self.__segment_1.line.end)
                if dist_begin>dist_end:
                    ''' удленняем дальше конечной точки'''
                    geometry_1_update = Line(self.__segment_1.line.begin,circle_line1_intrsect,
                                             cs=self.__segment_1.line.coordsystem)
                else:
                    geometry_1_update = Line(circle_line1_intrsect,self.__segment_1.line.end,
                                             cs=self.__segment_1.line.coordsystem)
        geometry_1_update=geometry_1_update.reproject(self.__coordsys_1)
        return geometry_1_update,None,split_circle_out
    def __BoundedIntersect(self,radius):
        '''
        q_line_point_select=QLineF(self.__segment_1.pnt_select.x,self.__segment_1.pnt_select.y,self.__segment_2.pnt_select.x,self.__segment_2.pnt_select.y)
        result_l1=self.__virtual_line_1_qt.intersect(q_line_point_select)
        result_l2 = self.__virtual_line_2_qt.intersect(q_line_point_select)
        '''
        temp_poly = LineString([PointToPnt(self.__segment_1.pnt_select), PointToPnt(self.__point_intersect),PointToPnt(self.__segment_2.pnt_select)])
        line_direction = temp_poly.get_line_direction()
        new_line1=Line(PointToPnt(self.__segment_1.pnt_select),PointToPnt(self.__point_intersect))
        new_line2 = Line(PointToPnt(self.__segment_2.pnt_select), PointToPnt(self.__point_intersect))
        if line_direction == LineDirection.Clockwise:
            line_1_parallel_p = shapaly_offset(new_line1, radius, TypeOffset.Rigth)
            line_2_parallel_p = shapaly_offset(new_line2, radius, TypeOffset.Left)
            line_1_parallel = line_1_parallel_p[1]
            line_2_parallel = line_2_parallel_p[0]
        else:
            line_1_parallel_p = shapaly_offset(new_line1, radius, TypeOffset.Left)
            line_2_parallel_p = shapaly_offset(new_line2, radius, TypeOffset.Rigth)
            line_1_parallel = line_1_parallel_p[0]
            line_2_parallel = line_2_parallel_p[1]
        qline_1 = toQLinef(line_1_parallel)
        qline_2 = toQLinef(line_2_parallel)
        intersect_info = qline_1.intersect(qline_2)
        point_intersect = Pnt(intersect_info[1].x(), intersect_info[1].y())
        rect_circle = Rect(point_intersect.x - radius, point_intersect.y - radius, point_intersect.x + radius,
                           point_intersect.y + radius)
        circl = Ellipse(rect_circle)
        circl.center = (point_intersect.x, point_intersect.y)
        circl.majorSemiAxis = radius
        circl.minorSemiAxis = radius
        poly_circle = circl.to_linestring()
        # circle_line1_intrsect=poly_circle.intersection(line_1p)
        circle_line1_intrsect = calcPerpendicularFromPoint(self.__segment_1.line, point_intersect)
        # circle_line2_intrsect = poly_circle.intersection(line_2p)
        circle_line2_intrsect = calcPerpendicularFromPoint(self.__segment_2.line, point_intersect)
        line_split_circle = Line(circle_line1_intrsect, circle_line2_intrsect)
        bound1=line_split_circle.bounds
        bound1=bound1.merge(self.__segment_1.line.bounds)
        bound1 = bound1.merge(self.__segment_2.line.bounds)
        line_split_circle=getExtendLineToBound(bound1,line_split_circle)
        split_circle = poly_circle.split_by_polyline(line_split_circle)

        temp_polygon_ = Polygon(
            [circle_line1_intrsect, self.__point_intersect,circle_line2_intrsect])

        centroid_1=split_circle[0].to_polygon().centroid()
        if len(split_circle)>1:
            centroid_2 = split_circle[1].to_polygon().centroid()
            is_poly_2 = temp_polygon_.contains(centroid_2)
        else:
            is_poly_2=False
        is_poly_1=temp_polygon_.contains(centroid_1)

        '''
        split_circle_out = poly_circle.difference(temp_polygon_)
        '''
        split_circle_out=split_circle[0]
        if is_poly_2 and ( not is_poly_1):
            split_circle_out = split_circle[1]
        geometry_1_update=None
        geometry_2_update = None
        line_p2 = (self.__segment_1.pnt_select, self.__segment_2.pnt_select)
        if self.__segment_1.editable and self.__segment_1.geometry_type==axipy.cpp_core_geometry.Type.Line:
            ''' сегмент линия и редактируемый'''
            geometry_1_update=self.__createNewGeometryByLine(self.__segment_1,circle_line1_intrsect,line_p2)
        #if self.__segment_2.editable and self.__segment_2.geometry_type==axipy.cpp_core_geometry.Type.Line:
        #    ''' сегмент линия и редактируемый'''
        #    geometry_2_update=self.__createNewGeometryByLine(self.__segment_2,circle_line2_intrsect,line_p2)
        if self.__segment_1.editable:
            geometry_1_update=self.__createNewGeometryByLine(self.__segment_1,circle_line1_intrsect,line_p2)
            geometry_1_update = geometry_1_update.reproject(self.__coordsys_1)
        if self.__segment_2.editable:
            geometry_2_update = self.__createNewGeometryByLine(self.__segment_2, circle_line2_intrsect, line_p2)
            geometry_2_update = geometry_2_update.reproject(self.__coordsys_2)

        return geometry_1_update,geometry_2_update,split_circle_out
    def __createNewGeometryByLine(self,segment,cirle_line_intersect,line_p2):
        out_geometry=None
        #line_p2 = (self.__segment_1.pnt_select, self.__segment_2.pnt_select)
        intersect_1 = lineIntersect((segment.line.begin, self.__point_intersect), line_p2)
        intersect_2 = lineIntersect((segment.line.end, self.__point_intersect), line_p2)
        if intersect_1==QLineF.UnboundedIntersection:
            intersect_1=fixIntersect((segment.line.begin, self.__point_intersect), line_p2)
        #if (self.__segment_1.line.begin==self.__segment_2.line.begin) or (self.__segment_1.line.begin==self.__segment_2.line.end):
        if (segment.line.begin == self.__point_intersect) or (segment.line.end == self.__point_intersect):
            intersect_1=QLineF.BoundedIntersection
        if intersect_2==QLineF.UnboundedIntersection:
            intersect_2=fixIntersect((segment.line.end, self.__point_intersect), line_p2)
        '''
        if self.__typeIntersect==QLineF.BoundedIntersection:
            # сегменты пересекаются
            value_intersect = (intersect_1 == QLineF.UnboundedIntersection or intersect_1 == QLineF.NoIntersection) and (intersect_2 == QLineF.BoundedIntersection or intersect_2 == QLineF.UnboundedIntersection)
            value_intersect=(intersect_1==QLineF.UnboundedIntersection or intersect_1==QLineF.NoIntersection) and (intersect_2==QLineF.BoundedIntersection )
            print(value_intersect)
            #if (intersect_1==QLineF.UnboundedIntersection or intersect_1==QLineF.NoIntersection) and intersect_2==QLineF.BoundedIntersection:
            if value_intersect:
                #  сегменты пересекаются в узлах
                intersect_1=QLineF.BoundedIntersection
        '''
        if intersect_1 == QLineF.BoundedIntersection:
            ''' выходной ( изменяемый объект ) для первого сегмента от первой точки до точки пересечения'''
            out_geometry = Line(segment.line.begin, cirle_line_intersect,
                                     cs=self.__base_cs)
        if intersect_2 == QLineF.BoundedIntersection:
            ''' выходной ( изменяемый объект ) для первого сегмента от точки пресечения до конечной точки'''
            out_geometry = Line(cirle_line_intersect, segment.line.end,
                                     cs=self.__base_cs)
        if intersect_1 == QLineF.UnboundedIntersection and intersect_2 == QLineF.UnboundedIntersection:
            ''' необходимо продить исходный отрезок'''
            dist_begin = Geometry.distance_by_points(self.__point_intersect, segment.line.begin)
            dist_end = Geometry.distance_by_points(self.__point_intersect, segment.line.end)
            if dist_begin > dist_end:
                ''' удленняем дальше конечной точки'''
                out_geometry = Line(segment.line.begin, cirle_line_intersect,
                                         cs=self.__base_cs)
            else:
                out_geometry = Line(cirle_line_intersect, segment.line.end,
                                         cs=self.__base_cs)
        return out_geometry




