from PySide2.QtCore import QLineF
from axipy import Pnt, Geometry, Line, LineString, LineDirection, Rect, Ellipse, Polygon

from ru_axioma_gis_add_geo_tools.tools.parallel_linestring.create_parallel import shapaly_offset, TypeOffset


def calcPerpendicularFromPoint(line:Line,p3:Pnt):
    x1=line.begin.x
    y1=line.begin.y
    x2=line.end.x
    y2=line.end.y
    a=x2-x1
    b=y2-y1
    a_sq = a * a
    b_sq = b * b
    ab = a * b
    a_sq_plus_b_sq = a_sq + b_sq
    x3=p3.x
    y3=p3.y
    x = (a_sq * x3 + ab * (y3 - y1) + b_sq * x1) / a_sq_plus_b_sq
    y = (a_sq * y1 + ab * (x3 - x1) + b_sq * y3) / a_sq_plus_b_sq
    return Pnt(x,y)
def getLineOrientationToPoint(line,point:Pnt):
    dist_1,angle=Geometry.distance_by_points(line.begin,point)
    dist_2, angle = Geometry.distance_by_points(line.end, point)
    if dist_1>dist_2:
        return line,Line(line.begin,point),False
    return Line(line.end, line.begin),Line(line.end,point),True
def toQLinef(geometry:Geometry):
    source_geo=geometry
    if isinstance(geometry,Line):
        source_geo=geometry.to_linestring()
    count_point=len(source_geo.points)
    return QLineF(source_geo.points[0].x,source_geo.points[0].y,source_geo.points[count_point-1].x,source_geo.points[count_point-1].y)
class LinePairing:
    def __init__(self,geo_line1,geo_line2):
        self.__line1=geo_line1
        self.__line2=geo_line2
        self.__typeIntersect=QLineF.BoundedIntersection
        self.__prepare()
    def __prepare(self):
        line_1 = QLineF(self.__line1.begin.x, self.__line1.begin.y, self.__line1.end.x, self.__line1.end.y)
        line_2 = QLineF(self.__line2.begin.x, self.__line2.begin.y, self.__line2.end.x, self.__line2.end.y)
        result = line_1.intersect(line_2)
        self.__typeIntersect=result[0]
        if self.__typeIntersect==QLineF.NoIntersection:
            return
        self.__point_intersect=Pnt(result[1].x(),result[1].y())
        #if self.__typeIntersect==QLineF.UnboundedIntersection:
        #    '''Линии персекаются в пространстве'''
    def build(self,radius):
        if self.__typeIntersect==QLineF.UnboundedIntersection:
            return self.__UnboundedIntersect(radius)
        if self.__typeIntersect==QLineF.BoundedIntersection:
            return self.__BoundedIntersect(radius)
    def __BoundedIntersect(self,radius):
        triangle_1=Polygon([self.__line1.begin,self.__point_intersect,self.__line2.begin])
        direction_line= triangle_1.get_line_direction()
        reverse_line_1=False
        reverse_line_2=False
        temp_line_1=None
        temp_line_2=None
        if direction_line==LineDirection.Clockwise:
            '''линии в одном направлении'''
            triangle_2 = Polygon([self.__line1.end, self.__point_intersect, self.__line2.end])
            area_1=triangle_1.get_area()
            area_2=triangle_2.get_area()
            if area_1>area_2:
                ''' расматриваем отрезки 1 го треугольника'''
                temp_line_1=Line(self.__line1.begin,self.__point_intersect)
                temp_line_2 = Line(self.__line2.begin, self.__point_intersect)
            else:
                ''' расматриваем отрезки 2 го треугольника'''
                temp_line_1=Line(self.__line1.begin,self.__point_intersect)

        else:
            triangle_1 = Polygon([self.__line1.end, self.__point_intersect, self.__line2.begin])
            triangle_2 = Polygon([self.__line1.begin, self.__point_intersect, self.__line2.end])
            area_1 = triangle_1.get_area()
            area_2 = triangle_2.get_area()
            if area_1>area_2:
                ''' расматриваем отрезки 1 го треугольника'''
                pass
            else:
                ''' расматриваем отрезки 2 го треугольника'''
                pass

    def __UnboundedIntersect(self,radius):
        '''Линии пресекаются ,но не явно'''
        new_line1,line_1p,reverse_points_1=getLineOrientationToPoint(self.__line1,self.__point_intersect)
        new_line2,line_2p,reverse_points_2=getLineOrientationToPoint(self.__line2,self.__point_intersect)
        temp_poly=LineString([new_line1.begin,self.__point_intersect,new_line2.begin])
        line_direction=temp_poly.get_line_direction()
        if line_direction==LineDirection.Clockwise:
            line_1_parallel_p=shapaly_offset(new_line1,radius,TypeOffset.Rigth)
            line_2_parallel_p = shapaly_offset(new_line2, radius, TypeOffset.Left)
            line_1_parallel=line_1_parallel_p[1]
            line_2_parallel=line_2_parallel_p[0]
        else:
            line_1_parallel_p = shapaly_offset(new_line1, radius, TypeOffset.Left)
            line_2_parallel_p = shapaly_offset(new_line2, radius, TypeOffset.Rigth)
            line_1_parallel = line_1_parallel_p[0]
            line_2_parallel = line_2_parallel_p[1]
        #point_intersect=line_1_parallel.intersection(line_2_parallel)
        qline_1=toQLinef(line_1_parallel)
        qline_2 = toQLinef(line_2_parallel)
        intersect_info=qline_1.intersect(qline_2)
        point_intersect=Pnt(intersect_info[1].x(),intersect_info[1].y())
        rect_circle=Rect(point_intersect.x-radius,point_intersect.y-radius,point_intersect.x+radius,point_intersect.y+radius)
        circl=Ellipse(rect_circle)
        circl.center=(point_intersect.x,point_intersect.y)
        circl.majorSemiAxis=radius
        circl.minorSemiAxis=radius
        poly_circle=circl.to_linestring()
        #circle_line1_intrsect=poly_circle.intersection(line_1p)
        circle_line1_intrsect=calcPerpendicularFromPoint(line_1p,point_intersect)
        #circle_line2_intrsect = poly_circle.intersection(line_2p)
        circle_line2_intrsect = calcPerpendicularFromPoint(line_2p,point_intersect)
        line_split_circle=Line(circle_line1_intrsect,circle_line2_intrsect)
        split_circle=poly_circle.split_by_polyline(line_split_circle)
        temp_polygon_=Polygon([new_line1.begin,new_line1.end,circle_line1_intrsect,circle_line2_intrsect,new_line2.end,new_line2.begin])
        split_circle_out=poly_circle.difference(temp_polygon_)
        out_line_1=None
        if reverse_points_1:
            out_line_1=Line(circle_line1_intrsect,self.__line1.end)
        else:
            out_line_1 = Line(self.__line1.begin,circle_line1_intrsect)
        out_line_2=None
        if reverse_points_2:
            out_line_2=Line(circle_line2_intrsect,self.__line2.end)
        else:
            out_line_2 = Line(self.__line2.begin,circle_line2_intrsect)
        return out_line_1,out_line_2,split_circle_out






