import os

import axipy
from PySide2.QtCore import Signal
from PySide2.QtGui import QDoubleValidator
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget
from axipy import Style, StyleButton


class ToolPairing(QWidget):
    signal_run=Signal(dict)
    def __init__(self,params,parent=None):
        super().__init__(parent)
        ui_file = os.path.join(os.path.dirname(__file__), "widget_pairing1.ui")

        self.__ui = QUiLoader().load(ui_file, parent)
        self.__d_validator=QDoubleValidator()
        self.__d_validator.setBottom(0.0001)
        self.__ui.ln_radius.setValidator(self.__d_validator)
        self.__params_building=None
        style_line = Style.from_mapinfo("Pen (4, 59, 16711680)")
        self.__style_line_btn = StyleButton(style_line, parent=self.__ui)
        self.__ui.verticalLayout_style.addWidget(self.__style_line_btn)
        self.__ui.pb_run.clicked.connect(self.__run)
        self.__ui.grp_rounding.clicked.connect(self.__change_cheked_rounding)
        self.__ui.grp_bevel.clicked.connect(self.__change_cheked_bevel)
        self.__init_params(params)
    def __init_params(self,params):
        if params is None:
            return
        isPairing= params['active']=='pairing'
        self.__ui.grp_rounding.setChecked(isPairing)
        self.__ui.grp_bevel.setChecked(not isPairing)
        self.__ui.ln_radius.setText(str(params['pairing']['radius']))
        self.__ui.ln_dist1_1.setText(str(params['bevel']['dist_1']))
        self.__ui.ln_dist1_2.setText(str(params['bevel']['dist_2']))
        mi_style=params['style']
        self.__style_line_btn.style=Style.from_mapinfo(mi_style)





    @property
    def widget(self):
        return self.__ui
    @property
    def style_sergment(self):
        return self.__style_line_btn.style
    def setEnabledRun(self,value=True):
        self.__ui.pb_run.setEnabled(value)
    def setEditableSegmet(self,id_segmet,value=True,value_checked=True):
        if id_segmet==0:

            self.__ui.chb_edit_segment_1.setEnabled(value)
            self.__ui.chb_edit_segment_1.setChecked(value_checked)
        else:
            self.__ui.chb_edit_segmet_2.setEnabled(value)
            self.__ui.chb_edit_segmet_2.setChecked(value_checked)
    def setEditableSegmet2(self,value=True):
        self.__ui.chb_edit_segment_2.setEnabled(value)
    def __run(self):
        if self.__ui.grp_rounding.isChecked():
            radius_value=float(self.__ui.ln_radius.text())
            segmet_1_editable=self.__ui.chb_edit_segment_1.isChecked()
            segmet_2_editable = self.__ui.chb_edit_segmet_2.isChecked()
            self.__params_building = {'method': 'pairing', 'radius': axipy.UnitValue(radius_value, axipy.LinearUnits.m),
                                      'edit_seg_1': segmet_1_editable, 'edit_seg_2': segmet_2_editable}
        if self.__ui.grp_bevel.isChecked():
            dist_1= float(self.__ui.ln_dist1_1.text())
            dist_2 = float(self.__ui.ln_dist1_2.text())
            segmet_1_editable = self.__ui.chb_edit_segment_1.isChecked()
            segmet_2_editable = self.__ui.chb_edit_segmet_2.isChecked()
            self.__params_building = {'method': 'bevel', 'dist_1': axipy.UnitValue(dist_1, axipy.LinearUnits.m), 'dist_2': axipy.UnitValue(dist_2, axipy.LinearUnits.m),
                                      'edit_seg_1': segmet_1_editable, 'edit_seg_2': segmet_2_editable}
        self.__ui.pb_run.setEnabled(False)
        self.signal_run.emit(self.__params_building)

    def __change_cheked_rounding(self,value):
        if value:
            self.__ui.grp_bevel.setChecked(False)
        else:
            self.__ui.grp_bevel.setChecked(True)
    def __change_cheked_bevel(self,value):
        if value:
            self.__ui.grp_rounding.setChecked(False)
        else:
            self.__ui.grp_rounding.setChecked(True)
    def getParams(self):
        params={}
        params['active']='pairing'
        if self.__ui.grp_bevel.isChecked():
            params['active'] = 'bevel'
        radius_value = float(self.__ui.ln_radius.text())
        segmet_1_editable = self.__ui.chb_edit_segment_1.isChecked()
        segmet_2_editable = self.__ui.chb_edit_segmet_2.isChecked()
        params['pairing']={ 'radius':radius_value,
                                      'edit_seg_1': segmet_1_editable, 'edit_seg_2': segmet_2_editable}
        dist_1 = float(self.__ui.ln_dist1_1.text())
        dist_2 = float(self.__ui.ln_dist1_2.text())
        segmet_1_editable = self.__ui.chb_edit_segment_1.isChecked()
        segmet_2_editable = self.__ui.chb_edit_segmet_2.isChecked()

        params['bevel'] = {'dist_1': dist_1,
                                  'dist_2': dist_2,
                                  'edit_seg_1': segmet_1_editable, 'edit_seg_2': segmet_2_editable}
        params['style']=self.__style_line_btn.style.to_mapinfo()
        return params






