from PySide2.QtCore import QObject, Signal
from axipy import selection_manager, Polygon, MultiPolygon

from ru_axioma_gis_add_geo_tools.tools.orthogonalization.tools.add_helper import CacheTabUpdate
from ru_axioma_gis_add_geo_tools.tools.orthogonalization.tools.orthogonalize_polygon_axioma import polygon_orthogonalize


class CalcIntProcent:
    def __init__(self,count):
        self.__last_proc=-1
        self.__count=count
    def getValue(self,curent):
        cur_val=int((curent/self.__count)*100+0.5)
        if cur_val==self.__last_proc:
            return None
        self.__last_proc=cur_val
        return cur_val


class Orthogonalize(QObject):
    change_progress=Signal(int)
    def __init__(self,maxAngleChange = 15, skewTolerance = 15):
        super().__init__()
        self.__maxAngleChange=maxAngleChange
        self.__skewTolerance=skewTolerance
        self.__isCancel=False
    def run(self):
        count_selection=selection_manager.count
        cls_calc_proc=CalcIntProcent(count_selection)
        tab_select=selection_manager.get_as_table()
        tab_cache=CacheTabUpdate(tab_select)
        for i,ft in enumerate(tab_select.items()):
            if self.__isCancel:
                break
            value_proc=cls_calc_proc.getValue(i)
            if value_proc is not None:
                self.change_progress.emit(value_proc)
            geometry=ft.geometry
            if geometry is None:
                continue
            if not (isinstance(geometry,Polygon) or isinstance(geometry,MultiPolygon)):
                continue
            new_geometry=polygon_orthogonalize(geometry,self.__maxAngleChange,self.__skewTolerance)
            if new_geometry is not None:
                ft.geometry=new_geometry
                tab_cache.add(ft)
        self.change_progress.emit(100)
        tab_cache.end()
        tab_select.close()
    def setCancel(self,value=True):
        self.__isCancel=value


