from enum import Enum

import axipy
from PIL.ImageChops import offset
from PySide2.QtCore import QObject, Signal
from axipy import Geometry, Pnt, LinearUnits, Feature
from shapely import from_wkb,to_wkb
from shapely.constructive import offset_curve
from shapely import LineString
from shapely.lib import polygonize

from ru_axioma_gis_add_geo_tools.tools.add_helper.add_tools import CalcIntProcent
from ru_axioma_gis_add_geo_tools.tools.add_helper.tab_helper import CloneFeature


class TypeOffset(Enum):
    Left=1
    Rigth=2
    DoubleSide=3
def shapaly_offset(geometry,offset_value,type_offset):
    wkb_geo = geometry.to_wkb()
    s_geo = from_wkb(wkb_geo)
    new_geo_left=None
    new_geo_right=None
    if type_offset==TypeOffset.Left:
        new_geo_left = offset_curve(s_geo, offset_value)

    if type_offset == TypeOffset.Rigth:
        new_geo_right = offset_curve(s_geo, -offset_value)
    if type_offset == TypeOffset.DoubleSide:
        new_geo_right = offset_curve(s_geo, -offset_value)
        new_geo_left = offset_curve(s_geo, offset_value)
    new_axy_geo_left=None
    new_axy_geo_right=None
    if new_geo_left is not None:
        new_axy_geo_left = Geometry.from_wkb(to_wkb(new_geo_left), geometry.coordsystem)
    if new_geo_right is not None:
        new_axy_geo_right=Geometry.from_wkb(to_wkb(new_geo_right), geometry.coordsystem)
    return new_axy_geo_left,new_axy_geo_right
def shapely_to_polygons(geometrys,coordsys):
    sh_geometrys=[]
    for geo in geometrys:
        wkb_geo = geo.to_wkb()
        sh_geometrys.append(from_wkb(wkb_geo))
    new_geo=polygonize(sh_geometrys)
    new_axi=Geometry.from_wkb(to_wkb(new_geo), coordsys)
    return new_axi



def distance_from_lineunit_to_degree(geometry:Geometry,dist):
    bound=geometry.bounds
    result_x=Geometry.distance_by_points(Pnt(bound.xmin,bound.center.y),Pnt(bound.xmax,bound.center.y),geometry.coordsystem)
    result_y = Geometry.distance_by_points(Pnt(bound.center.x,bound.ymin),Pnt(bound.center.x,bound.ymax),geometry.coordsystem)
    # axipy.UnitValue((result_x[0]+result_y[0])/2,LinearUnits.m)
    dist_x=result_x[0]
    dist_y=result_y[0]
    dx=bound.width
    dy=bound.height
    dx_degre_m=dx/dist_x
    dy_degre_m=dy/dist_y
    mid_degree_m=(dx_degre_m+dy_degre_m)/2
    #dist_in_meters=LinearUnits.m.to_unit(dist.unit,dist.value)
    dist_in_meters =dist.unit.to_unit(LinearUnits.m, dist.value)
    dist_degree=mid_degree_m*dist_in_meters
    return dist_degree

def createParallel(geometry,params):

    if geometry.coordsystem.lat_lon:
        offset=distance_from_lineunit_to_degree(geometry,params.offset)
    else:
        unit_cs=geometry.coordsystem.unit
        offset=params.offset.unit.to_unit(unit_cs,params.offset.value)
    new_geometry_left,new_geometry_right=shapaly_offset(geometry,offset,params.type_offset)
    if params.close_ends:
        if new_geometry_left is not None and new_geometry_right is not None:
            new_start=new_geometry_right.points[0]
            new_geometry_left.points.insert(0,new_start)
            count=len( new_geometry_right.points)
            new_geometry_left.points.append( new_geometry_right.points[count-1])
            if params.create_polygon:
                polygon_axi=shapely_to_polygons([new_geometry_left,new_geometry_right],geometry.coordsystem)
                new_geometry_left=polygon_axi
                new_geometry_right=None
        else:

            if new_geometry_left is not None:
                new_start = geometry.points[0]
                new_geometry_left.points.insert(0, new_start)
                count = len(geometry.points)
                new_geometry_left.points.append(geometry.points[count - 1])
                if params.create_polygon:
                    polygon_axi = shapely_to_polygons([new_geometry_left,geometry], geometry.coordsystem)
                    new_geometry_left = polygon_axi
            else:
                new_start = geometry.points[0]
                new_geometry_right.points.insert(0, new_start)
                count = len(geometry.points)
                new_geometry_right.points.append(geometry.points[count - 1])
                if params.create_polygon:
                    polygon_axi = shapely_to_polygons([new_geometry_right, geometry], geometry.coordsystem)
                    new_geometry_left = polygon_axi
                    new_geometry_right = None


    return new_geometry_left,new_geometry_right

class ParamsOffset:
    def __init__(self,offset,type_offset,style_line,close_end=False,create_polygon=False,style_poly=None):
        self.offset=offset
        self.type_offset=type_offset
        self.close_ends=close_end
        self.create_polygon=create_polygon
        self.style_line=style_line
        self.style_poly=style_poly
        self.delete_source_ft=False
        self.copy_source_data=False

class CreateParallel(QObject):
    update_progresbar=Signal(int)
    def __init__(self,table_source,params,table_out):
        super().__init__()
        self.__tab=table_source
        self.__tab_out=table_out
        self.__params=params
        self.__isCancel=False
    def run(self):
        print("Старт процесс")
        new_features=[]
        ft_delete=[]
        active_style=self.__params.style_line
        if self.__params.create_polygon:
            active_style=self.__params.style_poly
        cls_procent_progress=CalcIntProcent(self.__tab.count())
        for i,ft in enumerate(self.__tab.items()):

            procent=cls_procent_progress.getValue(i+1)

            if procent is not None:

                self.update_progresbar.emit(procent)
            if self.__isCancel:
                break
            if not(isinstance(ft.geometry,axipy.Line) or isinstance(ft.geometry,axipy.LineString)):
                continue

            new_geometry_left,new_geometry_right=createParallel(ft.geometry,self.__params)
            if new_geometry_left is not None:
                if self.__params.copy_source_data:
                    ft=CloneFeature(ft,new_geometry_left,active_style)
                else:
                    ft=Feature(geometry=new_geometry_left,style=active_style)
                new_features.append(ft)
            if new_geometry_right is not None:
                if self.__params.copy_source_data:
                    ft = CloneFeature(ft, new_geometry_right, active_style)
                else:
                    ft = Feature(geometry=new_geometry_right,style=active_style)
                new_features.append(ft)
            '''
            if self.__params.delete_source_ft:
                ft_delete.append(ft.id)
            '''
        if len(new_features)>0:
            self.__tab_out.insert(new_features)
        '''
        if len(ft_delete)>0:
            print("delete")
            self.__tab.remove(ft_delete)
        '''

    def IsCancel(self):
        self.__isCancel=True


