import os

from PySide2.QtGui import QDoubleValidator
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QDialog
from axipy import StyleButton, Style, StyleGeometryType, LinearUnits

from AxyPy.CreateMwsFromXsd.xsdObj.mws import LineStyle
from EsriMxd.xml.mxd.mws_xsd import AreaStyle


class DlgParallelOffset(QDialog):
    def __init__(self,parent=None):
        super().__init__(parent)
        ui_file = os.path.join(os.path.dirname(__file__), "dlg_parallel.ui")
        # Загружаем файл ui
        self.__ui = QUiLoader().load(ui_file, parent)
        self.__initStyle()
        self.__initUnits()
        self.__ui.ln_value_dist.setValidator(QDoubleValidator(0,99999,8))
        self.__ui.rb_left.toggled.connect(self.__onClickedRb)
        self.__ui.rb_right.toggled.connect(self.__onClickedRb)
        self.__ui.rb_two_side.toggled.connect(self.__onClickedRb)
        self.__ui.chek_close_line.stateChanged.connect(self.__change_close_lines)

    def __initStyle(self):
        style_line=Style.for_geometry_type(StyleGeometryType.Linear)
        self.__style_line_btn=StyleButton(style_line, parent=self.__ui)
        self.__ui.verticalLayout_line.addWidget(self.__style_line_btn)
        area_style=Style.for_geometry_type(StyleGeometryType.Polygonal)
        self.__style_area_btn = StyleButton( area_style, parent=self.__ui)
        self.__style_area_btn.setEnabled(False)
        self.__ui.verticalLayout_area.addWidget(self.__style_area_btn)
    def __initUnits(self):
        for item in LinearUnits.items():
            if item[1]==LinearUnits.degree:
                continue
            self.__ui.cmb_units.addItem(item[1].localized_name)
    def __onClickedRb(self):
        radioButton = self.sender()
    def __change_close_lines(self):
        self.__ui.ch_create_area.setEnabled(self.__ui.chek_close_line.isChecked())
        self.__style_area_btn.setEnabled(self.__ui.chek_close_line.isChecked())


    def show(self):
        self.__ui.show()
    def exec_(self):
        self.__ui.exec_()