import os

import axipy
from PySide2.QtGui import QDoubleValidator
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QDialog, QWidget
from axipy import StyleButton, Style, StyleGeometryType, LinearUnits, UnitValue, Pnt, Line, Rectangle

from ru_axioma_gis_add_geo_tools.tools.add_helper.tab_helper import tabInMap, compareSchema
from ru_axioma_gis_add_geo_tools.tools.parallel_linestring.create_parallel import TypeOffset, ParamsOffset, \
    CreateParallel


def getUnits(name_unit):
    for item in LinearUnits.items():
        if item[1].localized_name==name_unit:
            return item[1]
    return None


class FromParallel(QWidget):
    def __init__(self,parent=None):
        QWidget.__init__(self,parent)
        ui_file = os.path.join(os.path.dirname(__file__), "form_parallel_1.ui")
        # Загружаем файл ui
        self.__ui = QUiLoader().load(ui_file, parent)
        self.__source_table=None
        self.__initStyle()
        self.__initUnits()
        self.__ui.ln_value_dist.setValidator(QDoubleValidator(0, 99999, 8))
        self.__ui.rb_left.toggled.connect(self.__onClickedRb)
        self.__ui.rb_right.toggled.connect(self.__onClickedRb)
        self.__ui.rb_two_side.toggled.connect(self.__onClickedRb)
        self.__ui.chek_close_line.stateChanged.connect(self.__change_close_lines)
        self.__ui.ch_create_area.stateChanged.connect(self.__change_create_polygon)
        self.__ui.ch_direction_line.stateChanged.connect(self.__view_direction_line)
        self.__ui.pb_run.clicked.connect(self.__run)
        self.__activ_map_view=None

    def __initStyle(self):
        #style_line = Style.for_geometry_type(StyleGeometryType.Linear)
        style_line=Style.for_geometry(Line(Pnt(22, 44), (100, 101)))
        self.__style_line_btn = StyleButton(style_line, parent=self.__ui)
        self.__ui.verticalLayout_line.addWidget(self.__style_line_btn)
        #area_style = Style.for_geometry_type(StyleGeometryType.Polygonal)

        area_style = Style.for_geometry(Rectangle(0, 0, 40, 20) )
        self.__style_area_btn = StyleButton(area_style, parent=self.__ui)
        self.__style_area_btn.setEnabled(False)
        self.__ui.verticalLayout_area.addWidget(self.__style_area_btn)


    def __initUnits(self):
        for item in LinearUnits.items():
            if item[1] == LinearUnits.degree:
                continue
            self.__ui.cmb_units.addItem(item[1].localized_name)


    def __onClickedRb(self):
        radioButton = self.sender()


    def __change_close_lines(self):
        self.__ui.ch_create_area.setEnabled(self.__ui.chek_close_line.isChecked())
        #self.__style_area_btn.setEnabled(self.__ui.chek_close_line.isChecked())
    def __change_create_polygon(self):
        self.__style_area_btn.setEnabled(self.__ui.ch_create_area.isChecked())
    def init(self):
        if self.__activ_map_view is not None:
            self.__activ_map_view.editable_layer_changed.disconnect(self.__change_editable_layer)
        self.__activ_map_view=axipy.view_manager.active
        self.__activ_map_view.editable_layer_changed.connect(self.__change_editable_layer)
        try:
            self.__update_out_table()
        except:
            pass
        self.__source_table=axipy.selection_manager.get_as_table()
        unit_coordsystem=self.__source_table.coordsystem.unit
        index=self.__ui.cmb_units.findText(unit_coordsystem.localized_name)
        self.__ui.cmb_units.setCurrentIndex(index)
    def __change_editable_layer(self):
        try:
            self.__update_out_table()
        except:
            pass
    def __update_out_table(self):
        editable_layer = axipy.view_manager.active.map.editable_layer
        if editable_layer is not None :
            if editable_layer.title!='Косметический слой':
                self.__ui.label_tab_out.setText("Объекты быдут добавлены в:"+editable_layer.data_object.name)
            else:
                self.__ui.label_tab_out.setText("Объекты быдут добавлены в:" + "Косметический слой")
        else:
            cosmetic_layer = axipy.view_manager.active.map.cosmetic
            cosmetic_layer.visible = True
            axipy.view_manager.active.map.editable_layer = cosmetic_layer
            self.__ui.label_tab_out.setText("Объекты быдут добавлены в:"+"Косметический слой")
        tab_out=axipy.view_manager.active.map.editable_layer.data_object
        if self.__source_table is None:
            self.__source_table=axipy.selection_manager.get_as_table()
        isEqSchema=compareSchema(tab_out,self.__source_table)
        self.__ui.ch_copy_data.setEnabled(isEqSchema)
    def __view_direction_line(self):
        layer_source_selection=tabInMap(axipy.view_manager.active.map,self.__source_table.base_table)
        layer_source_selection.linesDirectionVisibile=self.__ui.ch_direction_line.isChecked()

    def __run(self):
        '''
        type_offset=TypeOffset.Left
        if self.__ui.rb_right.isChecked():
            type_offset=TypeOffset.Rigth
        if self.__ui.rb_two_side.isChecked():
            type_offset=TypeOffset.DoubleSide
        value_offset=float(self.__ui.ln_value_dist.text())
        unit_name=self.__ui.cmb_units.currentText()
        unit_dist=getUnits(unit_name)
        unit_dist_value=UnitValue(value_offset,unit_dist)
        is_close_line=self.__ui.chek_close_line.isChecked()
        tab_out=axipy.view_manager.active.map.editable_layer.data_object
        params=ParamsOffset(unit_dist_value,type_offset,self.__style_line_btn.style,is_close_line,self.__ui.ch_create_area.isChecked(),self.__style_area_btn.style)
        '''
        self.__ui.pb_run.setEnabled(False)
        self.__ui.pb_cancel.setEnabled(True)
        tab_out = axipy.view_manager.active.map.editable_layer.data_object
        params=self.getParams()
        params.delete_source_ft=self.__ui.ch_del_source.isChecked()
        params.copy_source_data=self.__ui.ch_copy_data.isChecked()
        tab_source=self.__source_table
        self.__cls_run=CreateParallel(tab_source,params,tab_out)
        #self.__cls_run.run()
        def run_process(t: axipy.Task):
            self.__cls_run.run()

        self.__run_task = axipy.Task(run_process)
        self.__cls_run.update_progresbar.connect(self.__update_progess)
        self.__run_task.finished.connect(self.__finish_task)
        self.__run_task.start()
    def __finish_task(self):
        if self.__ui.ch_del_source.isChecked():
            sql_del="DELETE FROM "+"Выборка"
            tab_del=axipy.data_manager.query_hidden(sql_del)
            jkl=0
        self.__cls_run.update_progresbar.disconnect(self.__update_progess)

        try:
            self.__ui.pb_run.setEnabled(True)
            self.__ui.pb_cancel.setEnabled(False)
        except:
            pass

        self.__cls_run=None
    def __update_progess(self, value):
        # print(value)
        self.__ui.progressBar.setValue(value)
    def setParams(self,params:ParamsOffset):
        unit_dist_value=params.offset
        value=unit_dist_value.value
        unit_name_loc=unit_dist_value.unit.localized_name
        index=self.__ui.cmb_units.findText(unit_name_loc)
        self.__ui.cmb_units.setCurrentIndex(index)
        self.__ui.ln_value_dist.setText(str(value))
        self.__style_line_btn.style=params.style_line
        self.__style_area_btn.style=params.style_poly
        if params.type_offset==TypeOffset.Left:
            self.__ui.rb_left.setChecked(True)
        else:
            self.__ui.rb_left.setChecked(False)
        if params.type_offset == TypeOffset.Rigth:
            self.__ui.rb_right.setChecked(True)
        else:
            self.__ui.rb_right.setChecked(False)
        if params.type_offset == TypeOffset.DoubleSide:
            self.__ui.rb_two_side.setChecked(True)
        else:
            self.__ui.rb_two_side.setChecked(False)
        self.__ui.chek_close_line.setChecked(params.close_ends)
        if params.create_polygon:
            self.__ui.ch_create_area.setEnabled(True)
            self.__ui.ch_create_area.setChecked(params.close_ends)
    def getParams(self):
        type_offset = TypeOffset.Left
        if self.__ui.rb_right.isChecked():
            type_offset = TypeOffset.Rigth
        if self.__ui.rb_two_side.isChecked():
            type_offset = TypeOffset.DoubleSide
        value_offset = float(self.__ui.ln_value_dist.text())
        unit_name = self.__ui.cmb_units.currentText()
        unit_dist = getUnits(unit_name)
        unit_dist_value = UnitValue(value_offset, unit_dist)
        is_close_line = self.__ui.chek_close_line.isChecked()

        params = ParamsOffset(unit_dist_value, type_offset, self.__style_line_btn.style, is_close_line,
                              self.__ui.ch_create_area.isChecked(), self.__style_area_btn.style)
        return params

    def show(self):
        self.__ui.show()


    @property
    def widget(self):
        return self.__ui

